
import java.applet.*;
import java.awt.*;

/////////////////////////////////////////////////////////////////
public class GameManager extends Applet
implements Runnable {

  // animation variables
  static final int REFRESH_RATE = 80; // in ms
  Thread animation;
  Graphics offscreen;
  Image image;

  // game variables used when playing
  public static final int TRICYTREME_VALUE = 130;   // 130 points
  public static final int DUST_VALUE = 50; 				 // 50 points

  static final int MAX_LEVEL = 9;
  static final int MAX_ENERGY = 113;

  private int score;
  private int numGathered;               // num of tricytreme minerals gathered

  Image trsImages[] =  new Image[15];    	// 15 tricytreme Images
  Image explodeImages[] =  new Image[10]; // 10 explode Images
  Image dustImages[] = new Image[1];  		// 25 dust images
  Image shipImage;                        	// ship image

  ShipManager sManager;                   // refers to ship manager
  TricytremeManager tManager;            	// refers to tricytreme manager
	DustManager dManager;										// refers to the dust manager

  // state of game
  private boolean playing;            // if game playing
  private int screen;                 // which screen to show
  static final int INTRO = 0;         // intro screen
  static final int GAME_OVER = 1;     // game over screen

  // Gather 25 Tricytreme minerals until next level
  static final int GATHER_FOR_NEXT_LEVEL = 15;

  AudioClip expsound;                 // explosion sound

  // fonts
  Font smallFont = new Font("Helvetica",Font.BOLD,12);
  Font mediumFont = new Font("Helvetica",Font.BOLD,14);
  Font bigFont = new Font("Helvetica",Font.BOLD,18);

  FontMetrics fm;                      // use to center string

  int width, height;                   // applet dimensions

	int minNumOfMinerals = 1;

  int numOfMinerals = 5;             	// the number Of Minerals
	int numOfDust = 10;									// the number of dust particles
  																		// (i.e. #tricytremes on screen)
	int maxNumOfDust = 25;

  // strings
  String scoreString = "Score: ";
  String gameOverString = "  GAME OVER  ";
  String clickString = "Shift-Click to Continue";
  String tricytremeGatheredString = "Tricytreme Gathered: ";

  int stringWidth;
  String introString[] = new String[8];

  public void init() {
    showStatus("Loading Images -- WAIT!");
    setBackground(Color.black);        // applet background
    width = bounds().width;            // set applet dimensions
    height = bounds().height;
    loadImages();

    try {
      expsound = getAudioClip(getCodeBase(),"Explosion.au");
    }
    catch (Exception e) { }

	  tManager = new TricytremeManager(numOfMinerals,width,height,trsImages,explodeImages,this,expsound);
    dManager = new DustManager(numOfDust,maxNumOfDust, width, height, dustImages, explodeImages, this, expsound);

		Intersect[] targets = new Intersect[maxNumOfDust + numOfMinerals];
		Intersect[] tricytremes = tManager.getTricytreme();
		Intersect[] dusts = dManager.getDust();

		int index = 0;

		for(int i = 0; i < tricytremes.length; i++)
		{
			targets[i] = tricytremes[i];
		}

		for(int i = tricytremes.length; i < targets.length; i++)
		{
			targets[i] = dusts[index++];
		}

    sManager = new ShipManager(MAX_ENERGY,5,width,height,shipImage,targets,this);

    tManager.initialize(sManager);            // initialize ship parameters
    dManager.initialize(sManager);

    playing = false;                   				// not playing
    screen = INTRO;                    				// show intro screen

    image = createImage(width,height); 				// make offscreen buffer
    offscreen = image.getGraphics();
    offscreen.setFont(bigFont);        				// font for intro
    fm = offscreen.getFontMetrics(bigFont);		// font metrics
//    stringWidth = fm.stringWidth(alienLandingString);

    introString[0] = "You are Humanity's last hope! Destroy the greedo nuggets ";
    introString[1] = "by using the mouse to move your space ship.";
    introString[2] = "Click to fire missiles.  Do not forget to gather the trycetreme";
    introString[3] = "minerals.  They are the most pressious stones in the galaxy.";
    introString[4] = "Just move your ship to catch the minerals. ";
    introString[5] = "Watch out! Multiple collisions with greedos may be lethal.";
    introString[6] = "The energy bar at the top of the screen will show your ship's remaining life.";
    introString[7] = "Click to start Game";

 }

  // load all images used in game
  public void loadImages() {

    MediaTracker tracker = new MediaTracker(this);
    shipImage = getImage(getCodeBase(),"pix/gun.gif");
    tracker.addImage(shipImage,0);

    for (int i=0; i<trsImages.length; i++) {
      trsImages[i] = getImage(getCodeBase(), "pix/trs" + i + ".gif");
      tracker.addImage(trsImages[i],0);
		}

    for (int i=0; i<explodeImages.length; i++) {
      explodeImages[i] = getImage(getCodeBase(), "pix/exp" + i + ".gif");
      tracker.addImage(explodeImages[i],0);
    }

    for (int i=0; i<dustImages.length; i++) {
			dustImages[i] = getImage(getCodeBase(), "pix/blkgold_nugget_b.gif");
			tracker.addImage(dustImages[i], 0);
		}

    // wait for all images to finish loading //
    try {
      tracker.waitForAll();
    }  catch (InterruptedException e) {
    }

    // check for errors //
    if (tracker.isErrorAny()) {
      showStatus("Error Loading Images");
    }
    else if (tracker.checkAll()) {
      showStatus("Images successfully loaded");
    }
  }

  // initialize params for new game
  public void newGame() {
    score = 0;                         // no score
    numGathered = 0;                   // no tricytremes gathered
    sManager.newGame();                      // call newGame in managers
    tManager.newGame();
    dManager.newGame();
    offscreen.setFont(smallFont);      // set font in game
  }


  // handle mouse events //
  public boolean mouseMove(Event e,int x,int y) {
    if (playing) {
      sManager.moveShip(x);
    }
    return true;
  }

  public boolean mouseDrag(Event e,int x,int y) {
    if (playing) {
      sManager.moveShip(x);
    }
    return true;
  }

  public boolean mouseDown(Event e,int x,int y) {
    if (playing) {
      sManager.fireMissile(x);
    }
    else if (screen == INTRO) {        // start game for mouse
                                       //   down on intro screen
      playing = true;
      newGame();
    }
    else if (screen == GAME_OVER) {    // else go back to intro
      if (e.shiftDown()) {             //   if shift-click
				screen = INTRO;
      }
    }

    return true;
  }

  // start the Video Game Loop
  public void start() {
    showStatus("Starting Game!");
    animation = new Thread(this);
     if (animation != null) {
       animation.start();
     }
  }

  // update managers. only update ship if playing
  public void updateManagers() {
    if (playing) {
      sManager.update();
    }
    tManager.update();
    dManager.update();
  }



  // override update so it doesn't erase screen
  public void update(Graphics g) {
    paint(g);
  }

  // paint the applet depending on mode of game
  public void paint(Graphics g) {
    if (playing) {
      offscreen.setColor(Color.black);
      offscreen.fillRect(0,0,width,height);  // clear buffer

      // draw status info
      offscreen.setColor(Color.cyan);
      offscreen.drawString(scoreString+score,width - 300,13);
      offscreen.drawString(tricytremeGatheredString+numGathered,width - 300,27);

      // tell TricytremeManager and ShipManager to paint
      tManager.paint(offscreen);
      dManager.paint(offscreen);
      sManager.paint(offscreen);


      g.drawImage(image,0,0,this);
    }
    else if (screen == INTRO) {
      offscreen.setColor(Color.black);
      offscreen.fillRect(0,0,width,height);  // clear buffer

      offscreen.setFont(smallFont);
      offscreen.setColor(Color.cyan);
      offscreen.drawString(scoreString+score,width - 300,13);
      offscreen.drawString(tricytremeGatheredString+numGathered, width - 300,27);
      tManager.paint(offscreen);
      dManager.paint(offscreen);

      offscreen.setFont(bigFont);

      offscreen.setColor(Color.green);
//      offscreen.drawString(alienLandingString, (width-stringWidth)/2, height/6);

      offscreen.setColor(Color.magenta);
      offscreen.setFont(mediumFont);

      // draw instructions
      for (int i=0; i<introString.length-1; i++) {
				offscreen.drawString(introString[i],13,(3+i)*height/12);
      }
      offscreen.setColor(Color.green);
      offscreen.drawString(introString[7], (width-stringWidth)/2, height*11/12);

      g.drawImage(image,0,0,this);

    }
    else if (screen == GAME_OVER) {
      offscreen.setColor(Color.black);

      offscreen.fillRect(0,0,width,height);  // clear buffer

      // draw status info
      offscreen.setFont(smallFont);
      offscreen.setColor(Color.cyan);
      offscreen.drawString(scoreString+score,width - 300,13);
      offscreen.drawString(tricytremeGatheredString+numGathered, width - 300,27);

      tManager.paint(offscreen);
      dManager.paint(offscreen);
      sManager.paint(offscreen);

      offscreen.setFont(bigFont);

      offscreen.setColor(Color.red);
      offscreen.drawString(gameOverString, (width-stringWidth)/2, height/2);
      offscreen.setFont(mediumFont);
      offscreen.setColor(Color.green);
      offscreen.drawString(clickString, (width-stringWidth-17)/2, height*11/12);

      g.drawImage(image,0,0,this);


    }
  }

  // the Video Game Loop
  public void run() {
    while (true) {
      repaint();
      updateManagers();
      Thread.currentThread().yield();
      try {
				Thread.sleep (REFRESH_RATE);
      } catch (Exception exc) { };
    }
  }


  // stop animation
  public void stop() {
    showStatus("Game Stopped");
    if (animation != null) {
      animation.stop();
      animation = null;
    }
  }

  // increase score
  public void incrementScore(int value) {
    score += value;
  }

  // decrease score
  public void decrementScore() {
		score -= TRICYTREME_VALUE;
	}

  // tracks the number of tricytremes gathered. If the
  // num_gathered is divisible by GATHER_FOR_NEXT_LEVEL
  // decrement the numOfMinerals
  public void gathered() {
    numGathered++;
    if (numGathered % GATHER_FOR_NEXT_LEVEL == 0) {
      if(numOfMinerals != minNumOfMinerals)
      {
				tManager.setNumOfMinerals(--numOfMinerals);
			}
			else
			{
				tManager.setNumOfMinerals(minNumOfMinerals);
			}

			if(numOfDust != maxNumOfDust)
			{
				dManager.setNumOfDust(++numOfDust);
			}
			else
			{
				dManager.setNumOfDust(maxNumOfDust);
			}
		}
  }

  // handle game over
  public void gameOver() {
    if (playing) {
      playing = false;
      screen = GAME_OVER;
    }
  }
}


/////////////////////////////////////////////////////////////////
class BitmapSprite extends Sprite {
  protected int locx;
  protected int locy;

  // image dimensions
  protected int width,height;

  protected Image image;                   // the bitmap
  protected Applet applet;                 // the parent applet

  public BitmapSprite(Image i,Applet a) {
    locx = 0;
    locy = 0;
    image = i;
    applet = a;
    if (image != null) {
      width = image.getWidth(a); // get size of background
      height = image.getHeight(a);
    }
    restore();
  }


  public BitmapSprite(int x,int y,Image i,Applet a) {
    locx = x;
    locy = y;
    image = i;
    applet = a;
    if (image != null) {
      width = image.getWidth(a); // get size of background
      height = image.getHeight(a);
    }
    restore();
  }

  public void setSize(int w,int h) {
    width = w;
    height = h;
  }

  public void update() {

    // do nothing

  }

  public void paint(Graphics g) {
    if (visible) {
      g.drawImage(image,locx,locy,applet);
    }
  }
}


// Moveable interface. Note that the declarations public
// and abstract are optional. All methods specified in an
// interface are automatically public and abstract!

interface Moveable {
  public abstract void setPosition(int x, int y);
  public abstract void setVelocity(int x, int y);
  public abstract void updatePosition();
}


/////////////////////////////////////////////////////////////////
class BitmapLoop extends BitmapSprite implements Moveable{
  protected Image images[];       // sequence of bitmaps
  protected int currentImage;     // the current bitmap
  protected boolean foreground;   // are there foreground images?
  protected boolean background;   // is there background image?
  // constructor. Assumes that background image is already
  // loaded. (use MediaTracker)

  public BitmapLoop(int x,int y,Image b,Image f[],Applet a) {
    super(x,y,b,a);
    if (image != null) {           // if there's a background image
      background = true;
    }
    else {
      background = false;
    }

    images = f;
    currentImage = 0;
    if (images == null || images.length == 0) {
      foreground = false;          // nothing in images[]
    }
    else {
      foreground = true;
      if (!background) {               // if no background
				width = images[0].getWidth(a); // get size of images[0]
				height = images[0].getHeight(a);
      }
    }

  }



  // cycle currentImage if sprite is active, and there
  //   are foreground images
  public void update() {
    if (active && foreground) {
      currentImage = (currentImage + 1) % images.length;
    }
    updatePosition();
  }


  public void paint(Graphics g) {
    if (visible) {
      if (background) {
				g.drawImage(image,locx,locy,applet);
      }
      if (foreground) {
				try
				{
					g.drawImage(images[currentImage],locx,locy,applet);
				}
				catch(Exception e)
				{
				}
      }
    }
  }

  // implement moveable interface


  public void setPosition(int x,int y) {
    locx = x;
    locy = y;
  }
  protected int vx;
  protected int vy;

  public void setVelocity(int x,int y) {
    vx = x;
    vy = y;
  }

  // update position according to velocity
  public void updatePosition() {
    locx += vx;
    locy += vy;

  }
}