
import java.applet.*;
import java.awt.*;

/////////////////////////////////////////////////////////////////
public class ShipManager {

  private ShipSprite ship;               // your ship
  private int ship_width;               // width of ship
  private int ship_height;
  private MissileSprite missile;       // missile
  private int min_x,max_x;             // min and max x coords
                                       //   for ship movement
  private int ship_min_x,ship_max_x;
  private int mis_min_x,mis_max_x;
  private int ship_y;

  private int energy;

  private int maxEnergy;
  private int energyDec;

  private GameManager game;            // ptr to game manager
  static int width, height;            // applet dimensions

  static final int ENERGY_PER_HIT = 5; // energy used per hit

  static final int MISSILE_WIDTH = 3;
  static final int MISSILE_HEIGHT = 15;
  static final int MISSILE_SPEED = -27; // missile flies upward
  static final Color MISSILE_COLOR= Color.blue;

  public ShipManager(int maxEnergy,int energyDec,int width,int height,
		    						Image shipImage,Intersect target[],Applet applet) {
    this.maxEnergy = maxEnergy;
    this.energyDec = energyDec;
    this.width = width;
    this.height = height;
    ship = new ShipSprite(shipImage,applet,this);

    ship_width = shipImage.getWidth(applet)/2;
    ship_height = shipImage.getHeight(applet);

    ship_y = height - ship_height;
    min_x = ship_width;
    max_x = width - ship_width;
    ship_min_x = 0;
    ship_max_x = width - 2*ship_width;
    mis_min_x = min_x-2;
    mis_max_x = max_x-2;
    ship.setPosition(width/2-ship_width,ship_y);

 		missile = new MissileSprite(MISSILE_WIDTH,MISSILE_HEIGHT,
																MISSILE_COLOR,MISSILE_SPEED,
																height-ship_height+13,
																0,target);

    game = (GameManager)applet;              // set ptr to GameManager
  }

  // set parameters for the new game
  public void newGame() {
    ship.setPosition(width/2-ship_width,ship_y);
    ship.restore();
    energy = maxEnergy;
  }


  // move ship to the given x coordinate
  public void moveShip(int x) {
    if (x <= min_x) {
      ship.setPosition(ship_min_x,ship_y);
    }
    else if (x >= max_x) {
      ship.setPosition(ship_max_x,ship_y);
    }
    else {
      ship.setPosition(x-ship_width,ship_y);
    }
  }

  // fire missile from given x coordinate
  public void fireMissile(int x) {
    if (!missile.isActive()) {     // if missile sprite
                                   //   isn't active
      if (x <= min_x) {
				missile.init(mis_min_x);
      }
      else if (x >= max_x) {
				missile.init(mis_max_x);
      }
      else {
				missile.init(x-1);        // initialize missile
      }
    }
  }

  // update all the parameters associated with the
  //   ship. In this case, only the missile needs to move
  //   automatically.
  public void update() {
    missile.update();
  }


  // paint all sprites associated with ship
  // also paint status display for amount of energy left
  String energyString = "Energy";
  public void paint(Graphics g) {
    ship.paint(g);
    missile.paint(g);

    // display energy left
    g.setColor(Color.red);
    g.drawString(energyString,3,13);
    g.fillRect(0,17,energy,10);
  }

  // accessor function for ship
  public ShipSprite getShip() {
    return ship;
  }

  // get the y-coordinate of the ship
  public int getShipY() {
    return ship_y;
  }

  // Gather an tricytreme
  public void handleHit() {
    if (energy <= 0) {             // game over if energy <= 0
      game.gameOver();             // notify game manager
      ship.suspend();               // turn off sprites
      missile.suspend();
    }
    else
    {
			energy--;
		}
  }

  // Gather an tricytreme
  public void gather() {
		game.incrementScore(GameManager.TRICYTREME_VALUE);
  }

  public void setEnergyDec(int dec) {
    energyDec = dec;
  }
}

class ShipSprite extends BitmapSprite
	implements Moveable,Intersect {

	public static final int GATHER = 0;
	public static final int HIT = 1;

  protected ShipManager gManager;         // pointer to manager class

  public ShipSprite(Image i, Applet applet,ShipManager gManager) {
    super(i,applet);
    this.gManager = gManager;
  }

  // the following methods implement Moveable:
  public void setPosition(int x,int y) {
    locx = x;
    locy = y;
  }

  public void setVelocity(int x,int y) {
		// No Implementation
  }

  public void updatePosition() {
		// No Implementation
  }

  // the following methods implement Intersect:
  // compare bounding boxes
  public boolean intersect(int x1,int y1,int x2,int y2) {
    return visible && (x2 >= locx) && (locx+width >= x1)
      && (y2 >= locy) && (locy+height >= y1);
  }

  // tell applet to display the hit
  public void hit(int type) {
		if(type == HIT) {
			gManager.handleHit();
		}
		else {
			gManager.gather();
		}
  }
}

/////////////////////////////////////////////////////////////////
class RectSprite extends Sprite2D {

  protected int width, height;    // dimensions of rectangle

  public RectSprite(int w,int h,Color c) {
    locx = 0;
    locy = 0;
    width = w;
    height = h;
    color = c;
    restore();
  }

  public RectSprite(int x,int y,int w,int h,Color c) {
    locx = x;
    locy = y;
    width = w;
    height = h;
    color = c;
    fill = false;                 // default: don't fill
    restore();                    // restore the sprite
  }

  // provide implementation of abstract methods:

  public void update() {

    // does nothing

  }


  // check if sprite's visible before painting
  public void paint(Graphics g) {
    if (visible) {
      g.setColor(color);

      if (fill) {
				g.fillRect(locx,locy,width,height);
      }

      else {
				g.drawRect(locx,locy,width,height);
      }
    }
  }
}

/////////////////////////////////////////////////////////////////
class MissileSprite extends RectSprite  {
  protected int vy;           // velocity in y coordinate
  protected int start_y;      // starting y coord
  protected int stop_y;       // stop at y coord
  Intersect target[];

  public MissileSprite(int w,int h,Color c,int vy, int start_y,int stop_y, Intersect target[]) {
    super(w,h,c);
    setFill(true);            // fill rect sprite
    this.vy = vy;             // initialize speed
    this.start_y = start_y;   // initialize starting point
    this.stop_y = stop_y;     // initialize stopping point
    this.target = target;     // initialize targets
    suspend();
  }

  // start the missile at the given x coordinate
  public void init(int x) {
    locx = x;
    locy = start_y;
    restore();
  }

  public void update() {

    if (active) {

      // move missile
      locy += vy;
      if (locy < stop_y) {
				suspend();
      }

      // else if missile hits target, suspend it
      else {
				for (int i=0; i<target.length; i++) {
	  			if (target[i].intersect(locx,locy,locx+width,locy+height)) {

	    			target[i].hit(0);  // tell target it's been hit

	    			suspend();
	    			break;
	  			}
				}
      }
    }
  }
}

interface Intersect {
  public boolean intersect(int x1,int y1,int x2,int y2);
  public void hit(int type);
}